/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.editor;

import java.awt.event.FocusListener;
import java.awt.event.KeyListener;

import javax.swing.event.DocumentListener;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;


/**
 * Gets notified about edit events:
 * <ul>
 * <li>{@link #editingStopped(EditingEvent)} - a request to stop the editing and commit the value from the editor.
 * <b>Unless a {@link #editingOccured()} is received the value will not be committed.</b></li>
 * <li>{@link #editingOccured()} - signals an edit event inside the editor. This will mark
 * the value from the editor as being dirty and requiring committing.</li>
 * <li>{@link #editingCanceled()} - a request to hide the editor without any commit.</li>
 * </ul>
 * 
 * An editor implementation will have to add listeners onto itself like:
 * <ul>
 *  <li>a {@link KeyListener} for handling key events like: ENTER to stop editing and ESCAPE to cancel it.</li>
 *  <li>a {@link FocusListener} to stop editing when the focus is given to a component that is not 
 *  part of the editor.</li>
 *  <li>a {@link DocumentListener} to fire {@link #editingOccured()} events (If the editor has a document).</li>
 * </ul>
 * @since 14.1 
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public interface InplaceEditingListener extends InplaceEditingTraversalListener {
  /**
   * An editing stopped request. This will commit the value into the document <b>ONLY
   * if the following conditions apply</b>:<ul>
   * <li>an {@link #editingOccured()} event was received prior to this event.</li>
   * <li>the value that must be committed is different from the old value. The old
   * value taken into account is either {@link InplaceEditorArgumentKeys#INITIAL_VALUE} or, if missing,
   * {@link InplaceEditorArgumentKeys#DEFAULT_VALUE}.</li>
   * <br/>
   * <b>OBS:</b> Before or after firing this event, the editor should release
   * any held resources. For example a SWT editor will have to dispose() any 
   * created images, fonts or controls. 
   * 
   * @param event Provides information about the editing. If <code>null</code> 
   * we should handle this as a cancel event.
   */
  void editingStopped(EditingEvent event);
  /**
   * An editing canceled request.
   * 
   * <br/>
   * <b>OBS:</b> Before or after firing this event, the editor should release
   * any held resources. For example a SWT editor will have to dispose() any 
   * created images, fonts or controls. 
   */
  void editingCanceled();
  /**
   * An edit happened in the inplace editor which could result in a document modification if the 
   * new value will be committed.
   * 
   * OBS: THIS EVENT IS VERY IMPORTANT. If no {@link #editingOccured()} event
   * is received, the value from the editor will not be committed when the editing 
   * is stopped. See {@link #editingStopped(EditingEvent)} for more information.
   */
  void editingOccured();
  
  /**
   * Commit the given value inside the document without stopping the editing. 
   * Will only commit if a new string value is provided and only if the value 
   * that must be committed is different from the current value.
   * 
   * Normally, this kind of event should be preceded by an {@link #editingOccured()} event.
   * 
   * @param event Editing event. Currently only the string value from within is of interest.
   * Also in case of custom form controls any given {@link EditingEvent#customEdit} will
   * also be executed.
   */
  void commitValue(EditingEvent event);
}