/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.webapp.formcontrols;

import java.util.HashMap;
import java.util.Map;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.component.editor.FormControlFactoryUtil;
import ro.sync.ecss.extensions.ExtensionLoader;
import ro.sync.ecss.extensions.api.Extension;
import ro.sync.ecss.extensions.api.ExtensionException;
import ro.sync.ecss.extensions.api.editor.AuthorInplaceContext;
import ro.sync.ecss.extensions.api.editor.InplaceEditorArgumentKeys;
import ro.sync.ecss.extensions.api.editor.InplaceEditorCSSConstants;
import ro.sync.ecss.webapp.formcontrols.WebappButtonGroupRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappButtonRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappCheckboxRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappComboRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappDatePickerRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappPopupRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappTextAreaRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappTextFieldRenderer;
import ro.sync.ecss.webapp.formcontrols.WebappURLChooserRenderer;

/**
 * Registry for form control renderers used in the Author Reviewer Webapp.
 * 
 * @author alex_jitianu
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public class WebappFormControlRendererRegistry {
  /**
   * A map of renderer for each type of form control.
   */
  private static final Map<String, WebappFormControlRenderer> renderers = 
      new HashMap<String, WebappFormControlRenderer>();

  /**
   * The built-in renderers.
   */
  static {
    registerRenderer(InplaceEditorCSSConstants.TYPE_BUTTON, new WebappButtonRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_TEXT, new WebappTextFieldRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_TEXT_AREA, new WebappTextAreaRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_CHECKBOX, new WebappCheckboxRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_COMBOBOX, new WebappComboRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_DATE_PICKER, new WebappDatePickerRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_URL_CHOOSER, new WebappURLChooserRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_POPUP_SELECTION, new WebappPopupRenderer());
    registerRenderer(InplaceEditorCSSConstants.TYPE_BUTTON_GROUP, new WebappButtonGroupRenderer());
  }
  
  /**
   * A map of renderer for each class mentioned in the CSS.
   */
  private final Map<String, WebappFormControlRenderer> customRenderers = 
      new HashMap<String, WebappFormControlRenderer>();
  
  /**
   * Constructor.
   */
  public WebappFormControlRendererRegistry() {
  }
  
  /**
   * Registers a renderer.
   * 
   * @param type The type of the form control. 
   * @param renderer The class responsible with rendering of that type of control.
   */
  private static void registerRenderer(String type, WebappFormControlRenderer renderer) {
    renderers.put(type, renderer);
  }
  
  /**
   * Returns the renderer for the specified type of control.
   * 
   * @param context The type of the form-control.
   * 
   * @return the renderer for the specified type of control.
   */
  public WebappFormControlRenderer getRenderer(AuthorInplaceContext context) {
    WebappFormControlRenderer renderer = null;
    
    // Expand the TYPE attribute into the corresponding properties.
    String type = (String) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_TYPE);
    if (type == null) {
      // We have a custom renderer.
      String rendererClass = 
          (String) context.getArguments().get(InplaceEditorArgumentKeys.PROPERTY_WEBAPP_RENDERER_CLASS_NAME);
      
      // First check the cache.
      renderer = customRenderers.get(rendererClass);
      if (renderer == null) {
        // Get renderer from author extension
        Extension rendererCandidate = null;
        try {
          rendererCandidate = ExtensionLoader.createExtensionInstance(
              rendererClass, FormControlFactoryUtil.createExtensionClasspath(context));
        } catch (ExtensionException e) {
        }
        if (rendererCandidate instanceof WebappFormControlRenderer) {
          renderer = (WebappFormControlRenderer) rendererCandidate;
        }
        if (renderer != null) {
          customRenderers.put(rendererClass, renderer);
        }
      }
    } else {
      renderer = renderers.get(type);
    }
    return renderer;
  }
}
