/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api;

import java.io.IOException;
import java.net.URL;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * Provides access to workspace specific information and actions. 
 * 
 * @since 11.2 
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface Workspace extends WorkspaceUtilities {
  
  /**
   * Opens the file at the specified {@link URL} in a new editor.
   * 
   * @param url The URL of the file to be opened.
   * @return <code>true</code> if the operation has succeeded.
   */
  boolean open(URL url);
  
  /**
   * Opens the file at the specified {@link URL} in a new editor.
   * 
   * @param url         The URL of the file to be opened.
   * @param imposedPage The imposed page for opening the URL.
   *
   * @return <code>true</code> if the operation has succeeded.
   * 
   * @since 13
   */
  boolean open(URL url, String imposedPage);
  
  /**
   * Opens the file at the specified {@link URL} in a new editor by specifying an imposed page and an imposed content type.
   * The imposed content type is used only in the Oxygen standalone application, it is not used in the Author Component and Eclipse plugin applications.
   * 
   * @param url         The URL of the file to be opened.
   * @param imposedPage The imposed page for opening the URL. Can be <code>null</code> to perform the default behavior.
   * 
   * @param imposedContentType The imposed content type, one of the constants in the interface ro.sync.exml.editor.ContentTypes. <br>
   * This is useful if for example the URL does not have a file extension (maybe it is a CMS resource) but the caller of the API knows that it is XML.
   * In this case the caller can provide the "text/xml" imposed content type for it to avoid Oxygen asking what type of resource the URL is.<br>
   * Another use case is for DITA Map URLs without an extension. The caller can pass the "application/ditamap" content type value to the API.
   * Oxygen will ask the user where to open the DITA Map (DITA Maps Manager or the main editor) and will continue the open procedure. <br>
   * Can be <code>null</code> to perform the default behavior.
   *
   * @return <code>true</code> if the operation has succeeded.
   * 
   * @since 15.2
   * 
   * <br>
   * <br>
   * *********************************
   * <br>
   * EXPERIMENTAL - Subject to change
   * <br>
   * ********************************
   * </br>
   * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
   * comments about it, please let us know.</p> 
   */
  boolean open(URL url, String imposedPage, String imposedContentType);
  
  /**
   * Saves the content of all opened and unsaved editors.
   */
  void saveAll();
  
  /**
   * Closes the editor specified by the URL.
   * <p>
   * If the editor has unsaved content, the user will be given the opportunity to save it.
   * </p>
   * 
   * @param url The url of the editor to be closed.
   * @return <code>true</code> if the editor was successfully closed, and
   *         <code>false</code> if the editor could not be closed.
   */
  boolean close(URL url);
  
  /**
   * Closes all the editors.
   * <p>
   * If there are editors with unsaved content, the user will be given the opportunity to save them.
   * </p>
   * 
   * @return <code>true</code> if the editors were successfully closed, and
   *         <code>false</code> if the editors are still open.
   */
  boolean closeAll();
  
  /**
   * Delete the resource identified by the specified {@link URL}.
   * Currently supported protocols are:
   * <ul>
   * <li>file://</li>
   * <li>zip://</li>
   * <li>ftp://</li>
   * <li>sftp://</li>
   * <li>http://</li>
   * <li>https://</li>
   * </ul>
   * 
   * @param url The URL from where to delete a resource.
   * @throws IOException
   */
  void delete(URL url) throws IOException;
  
  /**
   * In a new file appeared as a child of a folder in the project, use this method to 
   * refresh the parent folder.
   * @param url The new resource
   */
  void refreshInProject(URL url);
  
  /**
   * This is available only in the standalone Oxygen version (not available in the Oxygen Eclipse plugin).<br/>
   * Create a new "Untitled" editor. The editor content is not saved on disk, this method is equivalent to using the "File->New" action.
   * @param extension The editor extension ("xml" or "dita" or "xsl" or "xsd", etc...)
   * @param contentType The content type which can take values like: "text/xml" or "text/xsl" or "text/xsd", etc...
   * If NULL, the content type will be automatically detected from the extension. 
   * @param content The XML content will be used to load the new editor from.
   * @return The URL of the created new editor.
   * 
   * 
   * @since 12.1
   */
  URL createNewEditor(String extension, String contentType, String content);
  
  /**
   * Check if the extension is used in the Oxygen stand alone or Eclipse plugin version.
   * 
   * @return <code>true</code> if this is the stand-alone Oxygen version or
   * <code>false</code> if it is the Oxygen Eclipse plug-in version.
   */
  boolean isStandalone();
  
  /**
   * Set a title on the parent frame. This is available only in the standalone Oxygen version (not available in the Oxygen Eclipse plugin).
   * If NULL, will reset to the default title.
   * 
   * @param parentFrameTitle The new title to set on the parent frame.
   * If NULL, will reset to the default title.
   * 
   * @since 12.1
   */
  void setParentFrameTitle(String parentFrameTitle);
}