/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.converter;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import ro.sync.exml.plugin.PluginDescriptor;
import ro.sync.xml.parser.ParserCreator;

/**
 *  Contain all formats.
 *
 *@author     dan
 *@created    October 14, 2002
 *@version    $Revision: 1.31 $
 */
public class ConversionManager {

	/**
	 *  The path to conversion.xml file.
	 */
	public static String CONFIG_RESOURCE = "etc/conversion.xml";

	/**
	 *  All available formats.
	 */
	private final List formats;

	private final Properties stringResources = new Properties();
	private final Properties defaultValues = new Properties();

	/**
	 *  Extract all formats from configuration file.
	 *
	 *@exception  IOException          Thrown when conversion.xml file not
	 *      found.
	 *@exception  ConversionException  Thrown when invalid format is found.
	 */
	public ConversionManager() throws IOException, ConversionException {
		ConversionPlugin plugIn = ConversionPlugin.getInstance();
		PluginDescriptor descriptor = plugIn.getDescriptor();
		File baseDir = descriptor.getBaseDir();
		String configResource = new File(baseDir, CONFIG_RESOURCE).getPath();

		Element xmlRoot;
		try {
			// Get root element from xml file.
			xmlRoot = getRootElement(configResource);
		} catch (Exception ex) {
		  ex.printStackTrace();
			throw new ConversionException(ex.getMessage());
		}

		// Seet labels and default values.
		createStringResources(xmlRoot);
		createDefaultValues(xmlRoot);

		// Get all formats node
		NodeList formatsNodeList = xmlRoot.getElementsByTagName("entries");
		this.formats = new Vector();
		// Put in list all formats.
		for (int i = 0; i < formatsNodeList.getLength(); i++) {
			Element current = (Element) formatsNodeList.item(i);
			ConversionFormat conversionFormat = new ConversionFormat(current);
			formats.add(conversionFormat);
		}
	}

	/**
	 *  Return all formats.
	 *
	 *@return    All formats.
	 */
	public List getFormats() {
		return formats;
	}

	/**
	 *  Gets a conversion format by name. If format not found then return null.
	 *
	 *@param  formatName  The format name.
	 *@return             The conversion format
	 */
	public ConversionFormat getFormatByName(String formatName) {
		ConversionFormat result = null;
		Iterator it = formats.iterator();
		while (it.hasNext()) {
			ConversionFormat currentFormat = (ConversionFormat) it.next();
			if (currentFormat.getName().equals(formatName)) {
				result = currentFormat;
			}
		}
		return result;
	}

	/**
	 *  Gets the stringResource attribute of the ConversionManager object
	 *
	 *@param  key  Description of Parameter
	 *@return      The stringResource value
	 */
	public String getStringResource(String key) {
		return stringResources.getProperty(key, key);
	}

	/**
	 *  Gets the defaultValueAsString attribute of the ConversionManager object
	 *
	 *@param  key  Description of Parameter
	 *@return      The defaultValueAsString value
	 */
	public String getDefaultValueAsString(String key) {
		return defaultValues.getProperty(key, key);
	}

	/**
	 *  Gets the defaultValueAsBoolean attribute of the ConversionManager object
	 *
	 *@param  key  Description of Parameter
	 *@return      The defaultValueAsBoolean value
	 */
	public Boolean getDefaultValueAsBoolean(String key) {
		return Boolean.valueOf(defaultValues.getProperty(key, "false").equalsIgnoreCase("true"));
	}

	/**
	 *  Open xml file and return xml root element. If an exception has occurs
	 *  then return null.
	 *
	 *@param  xmlFileResource                   The name of xml file. It must be
	 *      a qualified resource name.
	 *@return                                   The root element.
	 *@exception  IOException                   Thrown when xml file not found.
	 *@exception  ParserConfigurationException  Description of Exception
	 *@exception  SAXException                  Description of Exception
	 */
	private Element getRootElement(String xmlFileResource)
		throws IOException, ParserConfigurationException, SAXException {
		InputStream inputStream = new FileInputStream(xmlFileResource);

		if (inputStream == null) {
			// This is a mac problem when the hosting directory of the
			// application has a space inside its name.
			inputStream = new FileInputStream("plugins" + xmlFileResource);
		}
		InputSource is = new InputSource(inputStream);
		DocumentBuilder parser = ParserCreator.newDocumentBuilderFakeResolver();

		return parser.parse(is).getDocumentElement();
	}

	/**
	 *  Set the default values.
	 *
	 *@param  xmlRoot  The root element from xml file
	 */
	private void createDefaultValues(Element xmlRoot) {
		// Set default values.
		Element defaultsElement = (Element) xmlRoot.getElementsByTagName("defaults").item(0);
		NodeList defaults = defaultsElement.getElementsByTagName("defaultvalue");

		String key;
		String value;
		for (int i = 0; i < defaults.getLength(); i++) {
			key = ((Element) defaults.item(i)).getAttribute("key");
			value = ((Element) defaults.item(i)).getAttribute("value");
			defaultValues.setProperty(key, value);
		}
	}

	/**
	 *  Set labels for internationalization.
	 *
	 *@param  xmlRoot  The root element from xml file.
	 */
	private void createStringResources(Element xmlRoot) {
		// Get labels node list.
		Element labelsElement = (Element) xmlRoot.getElementsByTagName("labels").item(0);
		NodeList labels = labelsElement.getElementsByTagName("label");

		// Parse all labels.
		String key = "";
		String value = "";
		for (int i = 0; i < labels.getLength(); i++) {
			key = ((Element) labels.item(i)).getAttribute("key");
			value = ((Element) labels.item(i)).getAttribute("value");
			stringResources.setProperty(key, value);
		}
	}
}
